@@ -6,10 +6,9 @@ weights=NULL,replace=TRUE,ties=TRUE,...) {
 	  	   # check input variables
 
        if (any(Group%%1==0)==FALSE & is.null(Group)==FALSE){
-       	print("The variable Group should be integer valued")
-       	stop
+       	stop("The variable Group should be integer valued")
        	}
-         if (is.unsorted(Group)==TRUE) { print("The variable Group is not in ascending order. Please reorder the input (X,Tr,Y, Group) in ascending order of Group."); stop
+         if (is.unsorted(Group)==TRUE) { stop("The variable Group is not in ascending order. Please reorder the input (X,Tr,Y, Group) in ascending order of Groupp.")
          	 # ordg<-order(Group)
        	     # Group<-Group[ordg] ; Y<-Y[ordg]; Tr<-Tr[ordg] ; X<-as.matrix(as.matrix(X)[ordg,])
            	  }
@@ -26,7 +25,7 @@ weights=NULL,replace=TRUE,ties=TRUE,...) {
         	}
         Gmax <- length(unique(Group))#dim(table((Group)));
         if (Gmax==1){
-        	warning("There is only one group: same output of Match")
+        	warning("There is only one group or no group has been specified: same output of Match")
         	B<-Match(Y,Tr,X,estimand=estimand,M=1,exact=exact,caliper=caliper,
 weights=weights,replace=replace,ties=ties)
         	return(B)
@@ -38,7 +37,7 @@ weights=weights,replace=replace,ties=ties)
         # Split the data set according to the Group variable
         Y2   <- split(Y, Group)
         Tr2  <- split(Tr, Group)
-        X2   <- split(as.data.frame(X), Group);X2<-sapply(X2,as.matrix)
+        X2   <- split(as.data.frame(X), Group);X2<-sapply(X2,as.matrix,simplify=FALSE)
         W2  <- split(weights, Group)
         
         # Initialize results
@@ -153,18 +152,22 @@ weights=weights,replace=replace,ties=ties)
         B$orig.treated.nobs.by.group<-table(Group[Tr==1])
         
         # the original number of control observations by group in the dataset:
-        #B$orig.control.nobs.by.group<-table(Group[Tr==0])
+        B$orig.control.nobs.by.group<-table(Group[Tr==0])
         
-        # the number of dropped observations by group in the dataset:
+        # the number of dropped observations by group in the matched dataset:
         #B$orig.ndrops.by.group.after.within<-
-        B$orig.ndrops.by.group<-table(Group[B$index.dropped])
+         B$orig.ndrops.by.group<-table(Group[B$index.dropped])
+        # B$orig.dropped.nobs.by.group<-orig.ndrops.by.group[names(table(Group))]
+         orig.ndrops.by.group<-0*table(Group)+ B$orig.ndrops.by.group[names(table(Group))]
+         orig.ndrops.by.group[is.na(orig.ndrops.by.group)]<-0
+         B$orig.dropped.nobs.by.group<-orig.ndrops.by.group
         
-
+        B$version<-b$version
           
         # Some extra info
         #B$MatchLoopC = b$MatchLoopC
-        B$version = "matchwithin"
         #return output
-        class(B) <- "Match"
+        #class(B) <- c("Match","CMatch")
+        class(B)    <- c("CMatch","Match")
         return(B) 
       }
